package de.hsbo.fbv.bmg.geometry.simple;

import java.util.ArrayList;

import com.vividsolutions.jts.geom.Coordinate;

import de.hsbo.fbv.ogc.geometry.simple.Geometry;

/**
 * Erweiterung der Klasse Coordinate (aus der Klassenbibliothek Java Topology
 * Suite (JTS) 1.10 der Fa. VividSolutions).<br>
 * (C) siehe Lizenzen von Geotools <br>
 * Ergnzt die JTS-Klasse Coordinate um die Komponente m (OGC Simple Feature Architecture)
 * 
 * @author A.Zimmermann, HS Bochum, 2011
 * @version 0.5
 * 
 */
public class CoordinateM extends Coordinate {

	private static final long serialVersionUID = 1L;

	/** Attribut m : dient zur Speicherung optionaler Measure-Werte */
	public double m = Double.NaN;
	
	/** Attribut trace : optionale Referenz auf ein Bezugsobjekt 
	 * (z.B. Realweltobjekt oder ein komplexes geometrisches Gebilde, 
	 * von dem dieses Objekt ein Bestandteil ist) */
	Object trace = null;

	/**
	 * Konstruktor
	 * @param c : Coordinate-Objekt der Java Topology Suite (JTS)
	 */
	public CoordinateM(Coordinate c) {
		super(c);
		if (c instanceof CoordinateM)
			this.m = ((CoordinateM) c).m;
	}

	/**
	 * Konstruktor: xyz-Koordinate mit zustzlichem m-Wert
	 * @param x
	 * @param y
	 * @param z
	 * @param m
	 */
	public CoordinateM(double x, double y, double z, double m) {
		super(x, y, z);
		this.m = m;
	}

	/**
	 * Konstruktor: xy-Koordinate mit zustzlichem m-Wert
	 * @param x
	 * @param y
	 * @param m
	 */
	public CoordinateM(double x, double y, double m) {
		super(x, y);
		this.m = m;
	}

	/**
	 * Konstruktor: xy-Koordinate
	 * @param x
	 * @param y
	 */
	public CoordinateM(double x, double y) {
		super(x, y);
	}

	/**
	 * Bestimmt die Koordinatendimension. Der optionale m-Wert wird nicht 
	 * bercksichtigt.
	 * @return
	 */
	public int getCoordinateDimension() {
		if (Double.isNaN(z))
			return 2;
		else
			return 3;
	}

	/**
	 * Stellt fest, ob ein optionaler m-Wert fr die Koordinate vorliegt.
	 * @return
	 */
	public boolean isMeasured() {
		if (Double.isNaN(m))
			return false;
		else
			return true;
	}

	/**
	 * Klassenmethode:<br>
	 * Konvertiert ein Coordinate-Feld (JTS) in ein CoordinateM-Feld. 
	 * @param coords
	 * @return
	 */
	public static CoordinateM[] toCoordinateArray(Coordinate[] coords) {
		int n = coords.length;
		CoordinateM[] cms = new CoordinateM[n];
		for (int i = 0; i < n; i++)
			cms[i] = new CoordinateM(coords[i]);
		return cms;
	}

	/**
	 * 
	 * @param coords
	 * @return
	 */
	public static ArrayList<CoordinateM> toCoordinateList(Coordinate[] coords) {
		int n = coords.length;
		ArrayList<CoordinateM> cms = new ArrayList<CoordinateM>();
		for (int i = 0; i < n; i++)
			cms.add(new CoordinateM(coords[i]));
		return cms;
	}

	/**
	 * @author zimmermann
	 */
	public int compareTo(Object o) {
		return compareTo(o, getSortIndex());
	}

	/**
	 * Fhrt den Vergleich von zwei Koordinaten mit Angabe der Koordinaten-Komponente durch.
	 * @param o
	 * @param sidx
	 * @return
	 */
	public int compareTo(Object o, int sidx) {
		if (sidx == GeoModel.DEFAULT) {
			Coordinate co = (Coordinate) o;
			return super.compareTo(co);
		}

		else {
			CoordinateM co = (CoordinateM) o;
			if (sidx == GeoModel.X)
				return compare(this.x, co.x);
			else if (sidx == GeoModel.Y)
				return compare(this.y, co.y);
			else if (sidx == GeoModel.Z)
				return compare(this.z, co.z);
			else
				// if(idx==GeoModel.M)
				return compare(this.m, co.m);
		}
	}

	/**
	 * Interne Methode
	 * @param thisv
	 * @param v
	 * @return
	 */
	private static int compare(double thisv, double v) {
		if (thisv < v)
			return -1;
		else if (thisv == v)
			return 0;
		else
			// if(thisv>v)
			return 1;
	}

	/**
	 * Gibt die Komponente fr den Koordinaten-Vergleich an. 
	 * @return
	 */
	public static int getSortIndex() {
		return GeoModel.sidx;
	}

	/**
	 * Getter-Methode fr das Bezugsobjekt
	 * @return
	 */
	public Object getTrace() {
		return trace;
	}

	/**
	 * Setter-Methode fr das Bezugsobjekt
	 * @param trace
	 */
	public void setTrace(Object trace) {
		this.trace = trace;
	}

}
