package de.hsbo.fbv.bmg.geometry.simple;

import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import com.vividsolutions.jts.io.WKBWriter;

import de.hsbo.fbv.ogc.geometry.simple.Binary;
import de.hsbo.fbv.ogc.geometry.simple.Geometry;

/**
 * Implementierung des Interface Geometry auf Basis der Klassenbibliothek Java
 * Topology Suite (JTS) 1.10 der Fa. VividSolutions<br>
 * (C) siehe Lizenzen von Geotools<br>
 * GGeometry ist eine abstrakte Klasse und zugleich die Urklasse aller geometrischen 
 * Datentypen dieser Klassenbibliothek.
 * @author A.Zimmermann, HS Bochum, 2011
 * @version 0.5
 */
public abstract class GGeometry implements Geometry, Shape {

	/** Attribut shape : graphischer Reprsentant fr die Ausgabe mit Java 2D */
	Shape shape = null;
	
	/** Attribut trace : optionale Referenz auf ein Bezugsobjekt 
	 * (z.B. Realweltobjekt oder ein komplexes geometrisches Gebilde, 
	 * von dem dieses Objekt ein Bestandteil ist) */
	Object trace = null;

	/**
	 * Getter-Methode fr das Bezugsobjekt
	 * @return
	 */
	public Object getTrace() {
		return trace;
	}

	/**
	 * Setter-Methode fr das Bezugsobjekt
	 * @param trace
	 */
	public void setTrace(Object trace) {
		this.trace = trace;
	}

	/**
	 * Gibt den Geometry-Typ an.
	 * @return
	 */
	public String getGeometryType() {
		return toJtsGeom().getGeometryType();
	}

	/**
	 * Konvertiert eine Geometry-Objekt der Java Topology Suite (JTS) in den 
	 * passenden Datentyp dieser Klassenbilbliothek. 
	 * @param geom : Geometry-Objekt der JTS
	 * @return
	 */
	public static Geometry toSfaGeom(com.vividsolutions.jts.geom.Geometry geom) {
		try {
			return GeoModel.createGeometry(geom);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Abstrakte Methode:<br>
	 * Konvertiert dieses Geoobjekt in den passenden Datentyp der Java Topology Suite 
	 * (JTS). 
	 * @return
	 */
	public abstract com.vividsolutions.jts.geom.Geometry toJtsGeom();

	/**
	 * Klassenmethode:<br>
	 * Konvertiert das Geoobjekt geom in den passenden Datentyp der Java Topology Suite.
	 * (JTS).
	 * @param geom
	 * @return
	 */
	public static com.vividsolutions.jts.geom.Geometry toJtsGeom(Geometry geom) {
		try {
			return (GeoModel.createGeometry(geom)).toJtsGeom();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Bildet das Envelope dieses Geoobjektes.
	 * @return Envelope-Objekt der Java Topology Suite 
	 */
	public com.vividsolutions.jts.geom.Envelope toJtsEnvelope() {
		return this.toJtsGeom().getEnvelopeInternal();
	}

	// Methoden von Interface Geometry

	@Override
	public Binary asBinary() {
		return new ByteArray(new WKBWriter().write(toJtsGeom()));
	}

	@Override
	public String asText() {
		return toJtsGeom().toText();
	}

	@Override
	public Geometry boundary() {
		return toSfaGeom(toJtsGeom().getBoundary());
	}

	@Override
	public Geometry buffer(double distance) {
		return toSfaGeom(toJtsGeom().buffer(distance));
	}

	@Override
	public boolean contains(Geometry geom) {
		return toJtsGeom().contains(toJtsGeom(geom));
	}

	@Override
	public Geometry convexHull() {
		return toSfaGeom(toJtsGeom().convexHull());
	}

	@Override
	public abstract int coordinateDimension();

	@Override
	public boolean crosses(Geometry geom) {
		return toJtsGeom().crosses(toJtsGeom(geom));
	}

	@Override
	public Geometry difference(Geometry geom) {
		return toSfaGeom(toJtsGeom().difference(toJtsGeom(geom)));
	}

	@Override
	public int dimension() {
		return toJtsGeom().getDimension();
	}

	@Override
	public boolean disjoint(Geometry geom) {
		return toJtsGeom().disjoint(toJtsGeom(geom));
	}

	@Override
	public double distance(Geometry geom) {
		return toJtsGeom().distance(toJtsGeom(geom));
	}

	@Override
	public Geometry envelope() {
		return toSfaGeom(toJtsGeom().getEnvelope());
	}

	@Override
	public boolean equals(Geometry geom) {
		return toJtsGeom().equalsExact(toJtsGeom(geom), GeoModel.getEPS());
	}

	@Override
	public String geometryType() {
		return toJtsGeom().getGeometryType();
	}

	@Override
	public Geometry intersection(Geometry geom) {
		return toSfaGeom(toJtsGeom().intersection(toJtsGeom(geom)));
	}

	@Override
	public boolean intersects(Geometry geom) {
		return toJtsGeom().intersects(toJtsGeom(geom));
	}

	@Override
	public boolean is3D() {
		if (this.coordinateDimension() == 3)
			return true;
		else
			return false;
	}

	@Override
	public boolean isEmpty() {
		return toJtsGeom().isEmpty();
	}

	@Override
	abstract public boolean isMeasured();

	@Override
	public boolean isSimple() {
		return toJtsGeom().isSimple();
	}

	@Override
	abstract public Geometry locateAlong(double arg0);

	@Override
	abstract public Geometry locateBetween(double arg0, double arg1);

	@Override
	public boolean overlaps(Geometry geom) {
		return toJtsGeom().overlaps(toJtsGeom(geom));
	}

	@Override
	public boolean relate(Geometry geom, String de9im) {
		return toJtsGeom().relate(toJtsGeom(geom), de9im);
	}

	@Override
	public int spatialDimension() {
		return this.coordinateDimension();
	}

	@Override
	public int srid() {
		return toJtsGeom().getSRID();
	}

	@Override
	public Geometry symDifference(Geometry geom) {
		return toSfaGeom(toJtsGeom().symDifference(toJtsGeom(geom)));
	}

	@Override
	public boolean touches(Geometry geom) {
		return toJtsGeom().touches(toJtsGeom(geom));
	}

	@Override
	public Geometry union(Geometry geom) {
		return toSfaGeom(toJtsGeom().union(toJtsGeom(geom)));
	}

	@Override
	public boolean within(Geometry geom) {
		return toJtsGeom().within(toJtsGeom(geom));
	}

	// Methoden von Interface Shape

	public Rectangle getBounds() {
		return shape.getBounds();
	}

	public Rectangle2D getBounds2D() {
		return shape.getBounds2D();
	}

	public boolean contains(double x, double y) {
		return shape.contains(x, y);
	}

	public boolean contains(Point2D p) {
		return shape.contains(p);
	}

	public boolean intersects(double x, double y, double w, double h) {
		return shape.intersects(x, y, w, h);
	}

	public boolean intersects(Rectangle2D r) {
		return shape.intersects(r);
	}

	public boolean contains(double x, double y, double w, double h) {
		return shape.contains(x, y, w, h);
	}

	public boolean contains(Rectangle2D r) {
		return shape.contains(r);
	}

	public PathIterator getPathIterator(AffineTransform at) {
		return shape.getPathIterator(at);
	}

	public PathIterator getPathIterator(AffineTransform at, double flatness) {
		return shape.getPathIterator(at, flatness);
	}

	@Override
	public String toString() {
		return asText();
	}
}
