package de.hsbo.fbv.bmg.geometry.simple;

import java.awt.Shape;
import java.awt.geom.Path2D;
import java.util.ArrayList;

import de.hsbo.fbv.ogc.geometry.simple.Geometry;
import de.hsbo.fbv.ogc.geometry.simple.GeometryCollection;

/**
 * Implementierung des Interface GeometryCollection auf Basis der
 * Klassenbibliothek Java Topology Suite (JTS) 1.10 der Fa. VividSolutions.<br>
 * (C) siehe Lizenzen von Geotools <br>
 * 
 * @author A.Zimmermann, HS Bochum, 2011
 * @version 0.5
 */
public class GGeometryCollection extends GGeometry implements
		GeometryCollection {

	/** Attribut jtsGeomColl : Delegat-Objekt zum Zugriff auf die JTS */
	protected com.vividsolutions.jts.geom.GeometryCollection jtsGeomColl = null;

	/**
	 * Konstruktor
	 * @param geoms : Feld vom Typ GGeometry
	 * @throws Exception
	 */
	public GGeometryCollection(GGeometry[] geoms) throws Exception {
		com.vividsolutions.jts.geom.Geometry[] jtsgeoms = new com.vividsolutions.jts.geom.Geometry[geoms.length];
		for (int i = 0; i < geoms.length; i++) {
			jtsgeoms[i] = geoms[i].toJtsGeom();
			if (!jtsgeoms[i].isValid())
				throw new Exception("Object " + i + " is not valid!");
		}
		this.jtsGeomColl = GeoModel.FACTORY.createGeometryCollection(jtsgeoms);
		this.shape = getShape(geoms);
	}

	/**
	 * Interne Methode:<br>
	 * Bildet aus der vorliegenden Geometrie ein Shape-Objekt (Typ Path2D).
	 * @param geoms
	 * @return
	 */
	Shape getShape(GGeometry[] geoms) {
		Path2D path = new Path2D.Double();
		for (int i = 0; i < geoms.length; i++)
			path.append(geoms[i], false);
		return path;
	}

	/**
	 * Konvertiert dieses Collection-Objekt in ein Feld vom Typ Geometry.
	 * @return
	 */
	public Geometry[] toArray() {
		Geometry[] geoms = new Geometry[this.numGeometries()];
		for (int i = 0; i < this.numGeometries(); i++)
			geoms[i] = this.geometryN(i);
		return geoms;
	}

	/**
	 * Konvertiert dieses Collection-Objekt in eine ArrayList vom Typ Geometry.
	 * @return
	 */
	public ArrayList<Geometry> toList() {
		ArrayList<Geometry> list = new ArrayList<Geometry>();
		for (int i = 0; i < this.numGeometries(); i++)
			list.add(this.geometryN(i));
		return list;
	}

	// Implementierte Methoden des Interface GeometryCollection

	@Override
	public Geometry geometryN(int n) {
		return toSfaGeom(jtsGeomColl.getGeometryN(n));
	}

	@Override
	public int numGeometries() {
		return jtsGeomColl.getNumGeometries();
	}

	@Override
	public com.vividsolutions.jts.geom.Geometry toJtsGeom() {
		return this.jtsGeomColl;
	}

	@Override
	public int coordinateDimension() {
		int cdim = 3;
		Geometry[] geoms = toArray();
		for (int i = 0; i < geoms.length; i++)
			cdim = Math.min(cdim, geoms[i].coordinateDimension());
		return cdim;
	}

	@Override
	public boolean isMeasured() {
		boolean res = true;
		Geometry[] geoms = toArray();
		for (int i = 0; i < geoms.length; i++)
			res = res && geoms[i].isMeasured();
		return res;
	}

	@Override
	public Geometry locateAlong(double arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Geometry locateBetween(double arg0, double arg1) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String asText() {
		return jtsGeomColl.toText();
	}

	/**
	 * Konvertiert dieses Objekt in ein passendes JTS-Objekt.
	 * @return
	 */
	public com.vividsolutions.jts.geom.GeometryCollection getJtsGeometryCollection() {
		return jtsGeomColl;
	}

}
