package de.hsbo.fbv.bmg.geometry.simple;

import java.awt.Shape;
import java.awt.geom.Line2D;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.LineSegment;

import de.hsbo.fbv.ogc.geometry.simple.Line;
import de.hsbo.fbv.ogc.geometry.simple.Point;

/**
 * Implementierung des Interface Line auf Basis der Klassenbibliothek Java
 * Topology Suite (JTS) 1.10 der Fa. VividSolutions.<br>
 * (C) siehe Lizenzen von Geotools <br>
 * 
 * @author A.Zimmermann, HS Bochum, 2011
 * @version 0.5
 */
public class GLine extends GLineString implements Line {

	/**
	 * Konstruktor
	 * @param cs
	 * @param ce
	 * @throws Exception
	 */
	public GLine(CoordinateM cs, CoordinateM ce) throws Exception {
		super(new CoordinateM[] { cs, ce });
		this.shape = getShape(this.getJtsLineString().getCoordinates());
	}

	/**
	 * Konstruktor
	 * @param ps
	 * @param pe
	 * @throws Exception
	 */
	public GLine(GPoint ps, GPoint pe) throws Exception {
		super(new GPoint[] { ps, pe });
		this.shape = getShape(this.getJtsLineString().getCoordinates());
	}

	/**
	 * Interne Methode:<br>
	 * Bildet aus der vorliegenden Geometrie ein Shape-Objekt (Typ Line2D).
	 * @param coords
	 * @return
	 */
	private Shape getShape(Coordinate[] coords) {
		Line2D path = new Line2D.Double(coords[0].x, coords[0].y, coords[1].x,
				coords[1].y);
		return path;
	}

	/**
	 * Konvertiert dieses Line-Objekt in ein LineSegment-Objekt der Java Topology Suite.
	 * @return
	 */
	public LineSegment toJtsLineSegment() {
		return new LineSegment(this.getJtsLineString().getCoordinates()[0],
				this.getJtsLineString().getCoordinates()[1]);
	}

	/**
	 * Gibt den Punkt mit dem geringeren x-Wert aus.
	 * @return
	 */
	public GPoint getFirstXPoint() {
		if (startPoint().x() <= endPoint().x())
			return (GPoint) startPoint();
		else
			return (GPoint) endPoint();
	}

	/**
	 * Gibt den Punkt mit dem hheren x-Wert aus.
	 * @return
	 */
	public GPoint getLastXPoint() {
		if (startPoint().x() >= endPoint().x())
			return (GPoint) startPoint();
		else
			return (GPoint) endPoint();
	}
	
	/**
	 * Berechnet den gerichteten Abstand des Punktes p von dieser Linie.<br>
	 * Punkte p links der Strecke S-E haben einen positiven Abstand.<br>
	 * Punkte p rechts der Strecke S-E haben einen negativen Abstand.<br>
	 * Berechnung erfolgt mit dem Kreuzproduktes gem. Gl. 2.10 BMG
	 * @param p
	 * @return
	 */
	public double directedDistance(GPoint p) {
		double length = this.length();
		Point s = this.startPoint();
		Point e = this.endPoint();
		double lx = e.x()-s.x();
		double ly = e.y()-s.y();
		double px = p.x()-s.x();
		double py = p.y()-s.y();
		double distance = (lx * py - ly * px) / length;
		return distance;
		
	}

}
