package de.hsbo.fbv.bmg.geometry.simple;

import de.hsbo.fbv.bmg.geometry.simple.collections.GPointArray;
import de.hsbo.fbv.ogc.geometry.simple.Geometry;
import de.hsbo.fbv.ogc.geometry.simple.MultiPoint;

/**
 * Implementierung des Interface MultiPoint auf Basis der Klassenbibliothek Java
 * Topology Suite (JTS) 1.10 der Fa. VividSolutions.<br>
 * (C) siehe Lizenzen von Geotools <br>
 * 
 * @author A.Zimmermann, HS Bochum, 2011
 * @version 0.5
 */
public class GMultiPoint extends GGeometryCollection implements MultiPoint {

	/**
	 * Konstruktor
	 * @param geoms
	 * @throws Exception
	 */
	public GMultiPoint(GPoint[] geoms) throws Exception {
		super(geoms);
		com.vividsolutions.jts.geom.Point[] jtsgeoms = new com.vividsolutions.jts.geom.Point[geoms.length];
		for (int i = 0; i < geoms.length; i++) {
			jtsgeoms[i] = geoms[i].getJtsPoint();
			if (!jtsgeoms[i].isValid())
				throw new Exception("Object " + i + " is not valid!");
		}
		this.jtsGeomColl = GeoModel.FACTORY.createMultiPoint(jtsgeoms);
	}

	@Override
	public com.vividsolutions.jts.geom.MultiPoint toJtsGeom() {
		return (com.vividsolutions.jts.geom.MultiPoint) this.jtsGeomColl;
	}

	/** Noch nicht implementiert! */
	@Override
	public Geometry locateAlong(double arg0) {
		try {
			throw new Exception("not yet implemented");
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/** Noch nicht implementiert! */
	@Override
	public Geometry locateBetween(double arg0, double arg1) {
		try {
			throw new Exception("not yet implemented");
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Gibt alle Punkte der Menge points zurck, die im angegebenen Rechteck
	 * liegen.
	 * 
	 * @param points
	 * @param a
	 * @param b
	 * @return
	 */
	public static GPoint[] selectRectangle(GPoint[] points, GPoint a, GPoint b) {
		GPointArray pa = new GPointArray(points);
		pa = pa.selectRectangle(a, b);
		if(pa!=null)
			return pa.toArray();
		else
			return null;
	}

	/**
	 * Gibt die extremen Punkte des Feldes an, durch die die BoundingBox
	 * bestimmt ist. Die Punkte sind in der Reihenfolge W,S,E,N angegeben.
	 * 
	 * @param points
	 * @return
	 */
	public static GPoint[] getExtremePoints(GPoint[] points) {
		GPointArray pa = new GPointArray(points);
		return pa.getExtremePointsXY();
	}

}
