package de.hsbo.fbv.bmg.geometry.simple;

import de.hsbo.fbv.ogc.geometry.simple.Geometry;
import de.hsbo.fbv.ogc.geometry.simple.MultiPolygon;
import de.hsbo.fbv.ogc.geometry.simple.Point;

/**
 * Implementierung des Interface LineString auf Basis der Klassenbibliothek Java
 * Topology Suite (JTS) 1.10 der Fa. VividSolutions.<br>
 * (C) siehe Lizenzen von Geotools <br>
 * 
 * @author A.Zimmermann, HS Bochum, 2011
 * @version 0.5
 */
public class GMultiPolygon extends GGeometryCollection implements MultiPolygon {

	/**
	 * Konstruktor
	 * @param geoms
	 * @throws Exception
	 */
	public GMultiPolygon(GPolygon[] geoms) throws Exception {
		super(geoms);
		com.vividsolutions.jts.geom.Polygon[] jtsgeoms = new com.vividsolutions.jts.geom.Polygon[geoms.length];
		for (int i = 0; i < geoms.length; i++) {
			jtsgeoms[i] = (com.vividsolutions.jts.geom.Polygon) geoms[i]
					.getJtsPolygon();
			if (!jtsgeoms[i].isValid())
				throw new Exception("Object " + i + " is not valid!");
		}
		this.jtsGeomColl = GeoModel.FACTORY.createMultiPolygon(jtsgeoms);
	}

	@Override
	public com.vividsolutions.jts.geom.MultiPolygon toJtsGeom() {
		return (com.vividsolutions.jts.geom.MultiPolygon) this.jtsGeomColl;
	}

	@Override
	public double area() {
		return ((com.vividsolutions.jts.geom.MultiPolygon) this.jtsGeomColl)
				.getArea();
	}

	@Override
	public Point centroid() {
		return GeoModel
				.createPoint(((com.vividsolutions.jts.geom.MultiPolygon) this.jtsGeomColl)
						.getCentroid());
	}

	@Override
	public Point pointOnSurface() {
		return GeoModel
				.createPoint(((com.vividsolutions.jts.geom.MultiPolygon) this.jtsGeomColl)
						.getInteriorPoint());
	}

	/** Noch nicht implementiert! */
	@Override
	public Geometry locateAlong(double arg0) {
		try {
			throw new Exception("not yet implemented");
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/** Noch nicht implementiert! */
	@Override
	public Geometry locateBetween(double arg0, double arg1) {
		try {
			throw new Exception("not yet implemented");
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

}
