package de.hsbo.fbv.bmg.geometry.simple;

import java.awt.geom.Line2D;

import de.hsbo.fbv.ogc.geometry.simple.Geometry;
import de.hsbo.fbv.ogc.geometry.simple.Point;

/**
 * Implementierung des Interface Point auf Basis der Klassenbibliothek Java
 * Topology Suite (JTS) 1.10 der Fa. VividSolutions.<br>
 * (C) siehe Lizenzen von Geotools <br>
 * 
 * @author A.Zimmermann, HS Bochum, 2011
 * @version 0.5
 */
public class GPoint extends GGeometry implements Point, Comparable<Point> {

	/** Attribut jtsGeomColl : Delegat-Objekt zum Zugriff auf die JTS */
	protected com.vividsolutions.jts.geom.Point jtsPoint = null;

	/**
	 * Konstruktor mit bergabe der xyz-Werte und des m-Wertes.
	 * @param x
	 * @param y
	 * @param z
	 * @param m
	 * @throws Exception
	 */
	public GPoint(double x, double y, double z, double m) throws Exception {
		this.jtsPoint = GeoModel.FACTORY
				.createPoint(new CoordinateM(x, y, z, m));
		if (!this.jtsPoint.isValid())
			throw new Exception("Object not valid!");
		this.shape = new Line2D.Double(x, y, x, y);
	}

	/**
	 * Konstruktor mit bergabe eines CoordinateM-Objektes.
	 * @param cm
	 * @throws Exception
	 */
	public GPoint(CoordinateM cm) throws Exception {
		this.jtsPoint = GeoModel.FACTORY.createPoint(cm);
		if (!this.jtsPoint.isValid())
			throw new Exception("Object not valid!");
		this.shape = new Line2D.Double(cm.x, cm.y, cm.x, cm.y);
	}

	@Override
	public double m() {
		return ((CoordinateM) this.jtsPoint.getCoordinate()).m;
	}

	@Override
	public double x() {
		return jtsPoint.getCoordinate().x;
	}

	@Override
	public double y() {
		return jtsPoint.getCoordinate().y;
	}

	@Override
	public double z() {
		return jtsPoint.getCoordinate().z;
	}

	@Override
	public com.vividsolutions.jts.geom.Point toJtsGeom() {
		return this.jtsPoint;
	}

	@Override
	public int coordinateDimension() {
		return ((CoordinateM) getJtsPoint().getCoordinate())
				.getCoordinateDimension();
	}

	@Override
	public boolean isMeasured() {
		return ((CoordinateM) getJtsPoint().getCoordinate()).isMeasured();
	}

	/**
	 * Diese Methode gibt den Punkt zurck, falls sein m-Wert mit dem angegebenen Wert bereinstimmt,
	 * sonst wird ein leeres Objekt zurckgegeben.
	 */
	@Override
	public Geometry locateAlong(double m) {
		if (this.isMeasured() && this.m() == m)
			return this;
		else
			return null;
	}

	/**
	 * Diese Methode gibt den Punkt zurck, falls sein m-Wert zwischen den angegebenen Werten liegt,
	 * sonst wird ein leeres Objekt zurckgegeben.
	 */
	@Override
	public Geometry locateBetween(double ms, double me) {
		if (this.isMeasured() && this.m() >= ms && this.m() <= me)
			return this;
		else
			return null;
	}

	@Override
	public int compareTo(Point go) {
		GPoint ggo = GeoModel.createPoint(go);
		return this.getCoordinate().compareTo(ggo.getCoordinate());
	}

	// Zustzliche Methoden

	/**
	 * Vergleicht die Rangordnung der Koordinaten in der Koordinatenkomponente sidx.
	 * @param go
	 * @param sidx
	 * @return
	 */
	public int compareTo(Point go, int sidx) {
		GPoint ggo = GeoModel.createPoint(go);
		return this.getCoordinate().compareTo(ggo.getCoordinate(), sidx);
	}

	/**
	 * Setzt (nachtrglich) den m-Wert.
	 * @param m
	 */
	public void setM(double m) {
		((CoordinateM) this.jtsPoint.getCoordinate()).m = m;
	}

	/**
	 * Setzt (nachtrglich) den z-Wert.
	 * @param m
	 */
	public void setZ(double z) {
		this.jtsPoint.getCoordinate().z = z;
	}

	/**
	 * Gibt das Delegat-Objekt zurck.<br>
	 * Achtung: Das Objekt kann unbeabsichtigt verndert werden!
	 * @return
	 */
	public com.vividsolutions.jts.geom.Point getJtsPoint() {
		return jtsPoint;
	}
	
	/**
	 * Gibt das Coordinate-Objekt zurck.<br>
	 * Achtung: Das Objekt kann unbeabsichtigt verndert werden!
	 * @return
	 */
	public CoordinateM getCoordinate() {
		return (CoordinateM) jtsPoint.getCoordinate();
	}
	
}
