package de.hsbo.fbv.bmg.geometry.simple.demos;

import java.awt.BasicStroke;
import java.awt.Color;

import de.hsbo.fbv.bmg.geometry.simple.GLinearRing;
import de.hsbo.fbv.bmg.geometry.simple.GPoint;
import de.hsbo.fbv.bmg.geometry.simple.GPolygon;
import de.hsbo.fbv.bmg.geometry.simple.GeoModel;
import de.hsbo.fbv.bmg.graphics.viewer.simple.Layer;
import de.hsbo.fbv.bmg.graphics.viewer.simple.ShapeLayer;

/**
 * Generator fr SimpleGeometry-Objekte.
 * 
 * @author A.Zimmermann, HS Bochum, 2012
 * 
 */
public class OGCGeometryFactory {

	/**
	 * Erzeugt einen Layer mit 100 GPoint-Objekten, die zufallsverteilt ein
	 * Gebiet von 1000x1000 berdecken.
	 * 
	 * @return
	 */
	public static Layer createPoints() {
		ShapeLayer<GPoint> layer = new ShapeLayer<GPoint>();
		for (int i = 0; i < 100; i++) {
			layer.add(GeoModel.createPoint(Math.random() * 1000,
					Math.random() * 1000));
		}
		layer.setDrawColor(Color.RED);
		layer.setStroke(new BasicStroke(7.f, BasicStroke.CAP_SQUARE,
				BasicStroke.JOIN_BEVEL));
		return layer;
	}

	/**
	 * Erzeugt einen Layer, das ein Polygon mit zufallsverteilten Eckpunkten
	 * enthlt.
	 * 
	 * @return
	 */
	public static Layer createSurface() {
		ShapeLayer<GPolygon> layer = new ShapeLayer<GPolygon>();
		int n = 10;
		GPoint[] pts = new GPoint[n];
		double x0 = 500, y0 = 500, r = 300, e = 200;
		for (int i = 0; i < n; i++) {
			pts[i] = GeoModel.createPoint(
							x0 + r * Math.cos(2 * i * Math.PI / n)
									+ (Math.random()-0.5) * e,
							y0 + r * Math.sin(2 * i * Math.PI / n)
									+ (Math.random()-0.5) * e);
		}
		GLinearRing crv = (GeoModel.createLineString(pts)).toLinearRing();
		if (crv.isRing()) {
			GPolygon srf = GeoModel.createPolygon(crv, null);
			layer.setDrawColor(Color.GREEN);
			layer.add(srf);
			layer.setStroke(new BasicStroke(3.f, BasicStroke.CAP_SQUARE,
					BasicStroke.JOIN_BEVEL));
			layer.setFillColor(Color.YELLOW);
			return layer;
		} else
			return null;
	}

}
