package de.hsbo.fbv.ogc.geometry.simple;

/**
 * Ein ein-dimensionales geometrisches Objekt (Linie), dessen Figur durch eine geordnete Menge (Liste) 
 * von Punkten festgelegt ist.
 * Der Rand (boundary) einer Linie besteht aus seinen beiden Endpunkten.
 * Aus topologischer Sicht sind diese ebenfalls Bestandteil der Linie.
 * @see Dokument OGC 06-103r3
 * @author (Open Geospatial Consortium Inc., OGC)
 * @version 1.2
 */
public interface Curve extends Geometry {
	
	/**
	 * Berechnet die Lnge der Linie.
	 * @return :double
	 */
	public double length();
	
	/**
	 * Gibt den Anfangspunkt der Linie an.
	 * @return :Point
	 */
	public Point startPoint();
	
	/**
	 * Gibt den Endpunkt der Linie an.
	 * @return :Point
	 */
	public Point endPoint();
	
	/**
	 * Untersucht, ob die Linie geschlossen ist (->true). Anfangs- und Endpunkt 
	 * mssen gleich (identisch) sein.
	 * In diesem Fall ist der Rand der Linien eine leere Menge.
	 * @return :boolean
	 */
	public boolean isClosed();
	
	/**
	 * Untersucht, ob die Linie ein Ring ist (->true). Ein Ring liegt vor, 
	 * wenn die Linie sowohl geschlossen als auch 'simple' ist, d.h. keine Selbst-
	 * berschneidung enthlt.
	 * @return :boolean
	 */
	public boolean isRing();

}
