package de.hsbo.fbv.ogc.geometry.simple;

/**
 * Ein null-dimensionales geometrisches Objekt, dessen Lage im Raum mindestens durch eine 
 * x- und eine y-Koordinate gegeben ist. Zustzlich kann ein Punkt eine z- und eine 
 * m-Komponente enthalten.
 * Der Rand (boundary) eines Punktes ist eine leere Menge.
 * @see Dokument OGC 06-103r3
 * @author (Open Geospatial Consortium Inc., OGC)
 * @version 1.2
 */
public interface Point extends Geometry {
	
	/**
	 * Gibt die x-Koordinate des Punktes zurck.
	 * @return :double
	 */
	public double x();
	
	/**
	 * Gibt die y-Koordinate des Punktes zurck.
	 * @return :double
	 */
	public double y();
	
	/**
	 * Gibt die z-Koordinate des Punktes zurck.
	 * @return :double
	 */
	public double z();
	
	/**
	 * Gibt den m-Wert des Punktes zurck.
	 * @return :double
	 */
	public double m();
}
