package de.hsbo.fbv.ogc.geometry.simple;

/**
 * Ein zwei-dimensionales geometrisches Objekt (Flche), dessen Figur durch eine geordnete Menge (Liste) 
 * von Punkten festgelegt ist.
 * Der Rand (boundary) einer Linie besteht aus seinen beiden Endpunkten.
 * @see Dokument OGC 06-103r3
 * @author (Open Geospatial Consortium Inc., OGC)
 * @version 1.2
 */
public interface Surface extends Geometry {
	
	/**
	 * Berechnet den Flcheninhalt der Oberflche.
	 * @return :double
	 */
	public double area();
	
	/**
	 * Berechnet den mathematischen Schwerpunkt.
	 * @return :Point
	 */
	public Point centroid();
	
	/**
	 * Berechnet einen reprsentativen Punkt, der auf der Flce liegt.
	 * @return :Point
	 */
	public Point pointOnSurface();
	
}
